#include <cassert>

#include <iostream>
using std::endl;

#include <iomanip>
using std::setw;

#include <iostream>
using std::ostream;

void format(ostream &os, const double v[], int N)
{
	for (int i = 0; i < N; ++i)
	{
		os << ( i == 0 ? "/" : ( i == N-1 ? "\\" : "|" ));
			os << setw(4) << v[i];
		os << ( i == 0 ? " \\" : ( i == N-1 ? " /" : " |" )) << endl;
	}
}

void format(ostream &os, const double A[], int N, int M)
{
	for (int i = 0; i < N; ++i)
	{
		os << ( i == 0 ? "/" : ( i == N-1 ? "\\" : "|" ));
		for (int j = 0; j < M; ++j)
			os << setw(4) << A[M*i+j];
		os << ( i == 0 ? " \\" : ( i == N-1 ? " /" : " |" )) << endl;
	}
}

double determinant2(const double A[4])
{
	return A[0]*A[3] - A[1]*A[2];
}

double determinant3(const double A[9])
{
	return + A[0]*(A[1+3]*A[2+3*2] - A[1+3*2]*A[2+3*1])
	       - A[0+3*1]*(A[1+3*0]*A[2+3*2] - A[1+3*2]*A[2+3*0])
	       + A[0+3*2]*(A[1+3*0]*A[2+3*1] - A[1+3*1]*A[2+3*0]);		  
}

double determinant(const double A[], int N, int N1 = -1, int* row_index = NULL, int* column_index = NULL)
{
	if (N1 == -1) N1 = N;
	int default_index[N];
	int* ri;
	int *ci;
	if (row_index == NULL)
	{
		assert(N1 == N);
		//assert(column_index == NULL);
		for (int i = 0; i < N; ++i)
			default_index[i] = i;
		ri = default_index;
		ci = default_index;
	}
	else
	{
		ri = row_index;
		ci = column_index;
	}
	if (N1 == 1) return A[N*ri[0]+ci[0]];

	double sum = 0.0;
	for (int j = 0; j < N1; ++j)
	{
		int reduced_row_index[N];
		int reduced_column_index[N];
		for (int i = 1; i < N1; ++i)
			reduced_row_index[i-1] = ri[i];
		int k = 0;
		for (int i = 0; i < N1; ++i)
			if (i != j)
				reduced_column_index[k++] = ci[i];
		assert(k == N1-1);
		sum += (j%2 == 0 ? 1 : -1)*A[N*ri[0]+ci[j]]*determinant(A,N,N1-1,reduced_row_index,reduced_column_index);
	}
	return sum;
}

double inner_prod(const double u[], const double v[], int N)
{
	double sum = u[0]*v[0];
	for (int i = 1; i < N; ++i)
		sum += u[i]*v[i];
	return sum;
}

void outer_prod(double out[], const double u[], const double v[], int N, int M)
{
	for (int i = 0; i < N; ++i)
		for (int j = 0; j < M; ++j)
			out[M*i+j] = u[i]*v[j];
}

void multiply(double out[], const double inL[], const double inR[], int N, int K, int M)
{
	for (int i = 0; i < N; ++i)
		for (int j = 0; j < M; ++j)
		{
			out[M*i+j] = 0.0;
			for (int k = 0; k < K; ++k)
				out[M*i+j] += inL[M*i+k]*inR[K*k+j];
		}
}

void multiply_Rtranspose(double out[], const double inL[], const double inR[], int N, int K, int M)
{
	for (int i = 0; i < N; ++i)
		for (int j = 0; j < M; ++j)
			out[M*i+j] = inner_prod(inL+K*i,inR+K*j,K);
	
}
